package sita;

import org.sanskritlibrary.catalogs.ApplicationState;
import org.sanskritlibrary.catalogs.TextPoint;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferStrategy;
import java.util.ArrayList;

public class AnnotationCanvas extends Canvas implements MouseListener, KeyListener, FocusListener {

    private Color xorColor = new Color(255, 255, 64); // Color.black;
    private Color xorColorSearch = new Color(0, 255, 255);
    private Color xorColorAnnotations = new Color(255,255, 64);
    private Color markerColor = Color.green;
    public ArrayList<String> lines;
    private boolean allowMouseClick = false;
    private boolean justGotFocus = false;
    private int currentTopLine;
    private int numLinesInPage;
    private int fontsize;
    private int lineHeight;
    private int startLine = -1;
    private int startPixelOffset = -1;
    private int startCharIndex = -1;
    private int endLine = -1;
    private int endPixelOffset = -1;
    private int endCharIndex = -1;
    private FontMetrics metrics;
    public ArrayList<TextSelection> hits = new ArrayList<TextSelection>();     // Keep track of all hits
    public int currentHit = -1;
    ApplicationState appstate;
    TextPoint endOfLastAnnotation = null;

    public AnnotationCanvas(ApplicationState appstate, int x, int y, int width, int height) {
        this.appstate = appstate;
        this.setBounds(x, y, width, height);
        this.setBackground(Color.white);
        this.addMouseListener(this);
        this.addKeyListener(this);
        this.addFocusListener(this);
        this.metrics = new FontMetrics(this.getFont()) {};
    }

    @Override
    public void addNotify() {
        super.addNotify();
        this.createBufferStrategy(2);       // does this work?
    }

    public void setText(ArrayList<String> lines) {
        this.lines = lines;
    }


    public void setPanelFont(Font font) {
        setFont(font);
        metrics = new FontMetrics(font) {};
        repaint();
    }

    public void setHighlightColor (Color c) {
        xorColor = c;
        repaint();
    }

    @Override
    public void paint(Graphics g) {
        if (lines != null) {
            BufferStrategy b = getBufferStrategy();
            Graphics bg = b.getDrawGraphics();
            int x = 0;
            int width = this.getWidth();
            int height = this.getHeight();
            bg.clearRect(0, 0, width, height);

            bg.setFont(this.getFont());
            fontsize = getFont().getSize();
            lineHeight = fontsize + 2;
            numLinesInPage = height / lineHeight;
            int i = currentTopLine;
            if (i >= 0) {
                int y = fontsize;
                while (y < height && i<lines.size()) {
                    bg.drawString(lines.get(i), x, y);
                    i = i + 1;
                    y = y + getFont().getSize() + 2;
                }
            } else {
                System.out.format("Why is currentTopLine<0?\n", i);
            }

            highlightSelection(bg, startLine, startCharIndex, endLine, endCharIndex);

            // highlight all hits, if any, of the last search
            xorColor = xorColorSearch;
            for (TextSelection ts : hits) {
                if (ts.lineNumber >= startLine && ts.lineNumber <= endLine) {
                } else {
                    highlightSelection(bg, ts.lineNumber, ts.start, ts.lineNumber, ts.end);
                }
            }
            xorColor = xorColorAnnotations;
            markEndOfLastAnnotation(bg, endOfLastAnnotation, false);

            b.show();
        }
        g.dispose();
    }

    private void highlightSelection (Graphics g, int startRow, int startCol, int endRow, int endCol) {
        if (startRow == -1 && endRow == -1 ) {
            return;             // no brackets
        } else if ((endRow != -1 && endRow < currentTopLine) || startRow >= currentTopLine + numLinesInPage) {
            return;     // selection is not visible
        } else if (startRow != -1 && endRow == -1) {      // just draw the left bracket
            drawLeftBracket(g);
        } else {                         // some of selection is visible
            int _start = startRow < currentTopLine ? currentTopLine : startRow;
            int _end = endRow >= currentTopLine + numLinesInPage ? currentTopLine + numLinesInPage -1 : endRow;
            for (int i=_start; i<=_end; i++) {
                String line = lines.get(i);
                int x1 = 0;
                int x2 = line.length();
                if (i == _start) {
                    if (i == startRow) {
                        x1 = startCol;
                    } else {
                        x1 = 0;
                    }
                }
                if (i == _end) {
                    if (i == endRow) {
                        x2 = endCol;
                    }
                }
                highlightSection(g, i, x1, x2);
            }
        }
    }

    private void highlightSection(Graphics g, int linenum, int x1, int x2) {
        int startOffset = 0;
        int endOffset = 0;
        String line = lines.get(linenum);
        int y =  (linenum - currentTopLine) * lineHeight;
        if (x1 > 0) {
            startOffset = (int)metrics.getStringBounds(line, 0, x1, g).getWidth();
        }
        if (x2 < line.length() - 1) {
            endOffset = (int)metrics.getStringBounds(line, 0, x2+1, g).getWidth();     // endPixelOffset include right bracket.
        } else {
            endOffset = (int)metrics.getStringBounds(line, 0, line.length(), g).getWidth();
        }
        g.setXORMode(xorColor);
        g.fillRect(startOffset, y, endOffset - startOffset, lineHeight);
        g.setPaintMode();
    }

    private void drawLeftBracket(Graphics g) {
        if (startLine >= currentTopLine && startLine < currentTopLine + numLinesInPage) {
            g.setXORMode(xorColor);
            int y = startLine - currentTopLine;
            CharInfo ci = getCharInfo(g, startPixelOffset, startLine);
            startCharIndex = ci.charIndex;
            g.fillRect(ci.pixelOffset, y * lineHeight + 2, ci.charWidth, lineHeight);
            g.setPaintMode();
        }
    }

    public void markEndOfLastAnnotation (TextPoint tp) {        // for guessing beginning of next annotation
        if (tp != null) {
            Graphics g = _getGraphics();
            markEndOfLastAnnotation (g, tp, true);
        }
    }

    public TextPoint getStartOfCurrentAnnotation () {
        if (startLine != -1) {
            return new TextPoint(startLine, startCharIndex);
        } else {
            return null;
        }
    }


    private void markEndOfLastAnnotation (Graphics g, TextPoint tp, boolean show) {
        if (tp != null && tp.row >= currentTopLine && tp.row < currentTopLine + numLinesInPage) {
            char[] chars = lines.get(tp.row).toCharArray();
            Rectangle2D r = metrics.getStringBounds(chars, 0, tp.col, g);
            int y = tp.row - currentTopLine;
            CharInfo ci = getCharInfo(g, (int)r.getWidth(), tp.row);
            g.setXORMode(markerColor);
            g.fillRect(ci.pixelOffset, y * lineHeight + 2, 3, lineHeight);
            g.setPaintMode();
            if (show) {
                _show(g);
            }
            endOfLastAnnotation = tp;
        }
    }

    private void drawRightBracket(Graphics g) {
        if (endLine >= currentTopLine && endLine < currentTopLine + numLinesInPage) {
            g.setXORMode(xorColor);
            int y = endLine - currentTopLine;
            CharInfo ci = getCharInfo(g, endPixelOffset, endLine);
            endCharIndex = ci.charIndex;
            g.fillRect(ci.pixelOffset, y * lineHeight, 10, lineHeight);
            g.setPaintMode();
        }
    }

    private void removeLeftBracket (Graphics g) {
        drawLeftBracket(g);
        startLine = -1;
        startPixelOffset = -1;
        startCharIndex = -1;
    }

    private void removeSelection (Graphics g) {
        if (startLine != -1) {
            this.highlightSelection(g, startLine, startCharIndex, endLine, endCharIndex);             // xor will remove highlighting
            startLine = -1;
            startPixelOffset = -1;
            startCharIndex = -1;
            endLine = -1;
            endPixelOffset = -1;
            endCharIndex = -1;
        }
    }

    public void removeSelection () {
        Graphics g = _getGraphics();
        this.removeSelection(g);
        _show(g);
    }

    private CharInfo getCharInfo(Graphics g, int x, int y) {
        String line = lines.get(y);
        char[] chars = line.toCharArray();
        int prevWidth = 0;
        for (int i = 0; i < chars.length; i++) {
            Rectangle2D rect = metrics.getStringBounds(chars, 0, i, g);
            if (rect.getWidth() > x) {
                return new CharInfo(prevWidth, i - 1, (int)rect.getWidth() - prevWidth);
            } else {
                prevWidth = (int) rect.getWidth();
            }
        }

        return new CharInfo(prevWidth, chars.length - 1, 10);
    }

    private int getPixelOffsetFromIndex(Graphics g, int y, int idx) {
        String line = lines.get(y);
        Rectangle2D rect = null;
        if (idx < line.length()) {
            rect = metrics.getStringBounds(line.substring(0, idx), g);
        } else {
            System.out.format("%s: %d\n", line, idx);
        }
        return (int) rect.getWidth();
    }

    private int getNearestCharToPixel(Graphics g, int y, int x) {
        String line = lines.get(y);
        Rectangle2D rect = null;
        int len = line.length();
        int prevOffset = -1;
        int width = 0;
        for (int i=1; i<len; i++) {
            rect = metrics.getStringBounds(line.substring(0, i+1), g);
            width = (int)rect.getWidth();
            if (width > x) {
                if (prevOffset == -1) {
                    return 0;
                }
                else if (x - prevOffset > width - x)
                {
                    return i;
                }
                else {
                    return i-1;
                }
            }
            else {
                prevOffset = width;
            }
        }
        return len - 1;
    }

    private int getIndex (boolean drawLeft) {
        if (drawLeft) {
            return startCharIndex;
        } else {
            return endCharIndex;
        }
    }

    private void setIndex (boolean drawLeft, int idx) {
        if (drawLeft) {
            startCharIndex = idx;
        } else {
            endCharIndex = idx;
        }
    }

    private void incIndex (boolean drawLeft, int inc) {
        if (drawLeft) {
            startCharIndex = startCharIndex + inc;
        } else {
            endCharIndex = endCharIndex + inc;
        }
    }

    private int getPixelOffset (boolean drawLeft) {
        if (drawLeft) {
            return this.startPixelOffset;
        } else {
            return this.endPixelOffset;
        }
    }

    private void setPixelOffset (boolean drawLeft, int pixelOffset) {
        if (drawLeft) {
            startPixelOffset = pixelOffset;
        } else {
            endPixelOffset = pixelOffset;
        }
    }


    private int getLine (boolean drawLeft) {
        if (drawLeft) {
            return startLine;
        } else {
            return endLine;
        }
    }

    public String getLineText(int n) {
        if (n >=0 && n < lines.size()) {
            return lines.get(n);
        } else {
            return null;
        }
    }

    public void showLine (TextSelection ts) {
        this.makeLineViewable(ts.lineNumber, true);
        repaint();
    }

    public void removeMatchHighlights () {
        hits.clear();
        repaint();
    }

    public int getNumLines () {
        return lines.size();
    }


    private void incLine (boolean drawLeft, int inc) {
        if (drawLeft) {
            startLine = startLine + inc;
        } else {
            endLine = endLine + inc;
        }
    }

    public void setCurrentTopLine(int line) {
        currentTopLine = line;
        repaint();
    }

    public int nextPage() {
        currentTopLine = currentTopLine + numLinesInPage;
        if (currentTopLine > lines.size() - 1) {
            currentTopLine = lines.size() - 1;
        }
        repaint();
        return currentTopLine;
    }

    public int prevPage() {
        currentTopLine = currentTopLine - numLinesInPage;
        if (currentTopLine < 0) {
            currentTopLine = 0;
        }
        repaint();
        return currentTopLine;
    }

    public void mouseClicked(MouseEvent e) {
        if (appstate.getReviewOnly()) {
            Utils.displayReviewOnlyWarning();
            return;
        }
        if (justGotFocus) {
            justGotFocus = false;
            return;
        }
        int n = currentTopLine + e.getY() / lineHeight;
        Graphics g = null;
        for (int pass = 0; pass < 2; pass++) {
            if (startLine == -1) {
                startLine = n;
                startPixelOffset = e.getX();
                if (g == null) {
                    g = _getGraphics();
                }
                drawLeftBracket(g);
                break;
            } else if (endLine == -1) {
                endLine = n;
                endPixelOffset = e.getX();
                if (g == null) {
                    g = _getGraphics();
                }
                endCharIndex = getCharInfo(g, endPixelOffset, endLine).charIndex;
                if (startLine == endLine && startCharIndex == endCharIndex) {
                    removeLeftBracket(g);
                    endLine = -1;
                    endPixelOffset = -1;
                    endCharIndex = -1;
                    break;
                }
                if (pass == 0) {
                    drawLeftBracket(g);             // get rid of left bracket
                }
                this.highlightSelection(g, startLine, startCharIndex, endLine, endCharIndex);
                break;
            } else {        // both brackets exist, clicking on start bracket deletes both brackets, otherwise end bracket is moved to click location
                g = _getGraphics();
                if (n == startLine) {
                    CharInfo ci = getCharInfo(g, e.getX(), startLine);
                    if (ci.charIndex == startCharIndex) {
                        removeSelection(g);
                        break;
                    }
                }

                this.highlightSelection(g, startLine, startCharIndex, endLine, endCharIndex);     // Remove current highlighting so it can be redrawn
                endLine = -1;
                // do another interation
            }
        }
        if (g != null) {
            _show(g);
        }
    }

    private Graphics _getGraphics () {
        BufferStrategy b = getBufferStrategy();
        return b.getDrawGraphics();
    }

    private void _show (Graphics g) {
        getBufferStrategy().show();
        g.dispose();
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void keyTyped(KeyEvent e) {
    }

    public void keyPressed(KeyEvent e) {
        if (startLine == -1 && endLine == -1) {
            return;             // no brackets visible so nothing to do.
        }

        if (appstate.getReviewOnly()) {
            Utils.displayReviewOnlyWarning();
            return;
        }

        Graphics g = null;

        try {
            int kc = e.getKeyCode();
            if (kc == KeyEvent.VK_DELETE || kc == KeyEvent.VK_BACK_SPACE) {
                g = _getGraphics();
                if (endLine == -1) {
                    removeLeftBracket(g);
                } else {
                    removeSelection(g);
                }
            } else if (kc == KeyEvent.VK_LEFT || kc == KeyEvent.VK_RIGHT) {
                moveHorizontally(g = _getGraphics(), kc);
            } else if (kc == KeyEvent.VK_UP || kc == KeyEvent.VK_DOWN) {
                moveVertically(g = _getGraphics(), kc);
            }
        } finally {
            if (g != null) {
                _show(g);
            }
        }
    }

    // move left or right bracket left or right possibly going up or down a line
    public void moveHorizontally(Graphics g, int kc) {
        int inc = 0;
        if (kc == KeyEvent.VK_LEFT) {
            inc = -1;
        } else if (kc == KeyEvent.VK_RIGHT) {
            inc = 1;
        }
        boolean moveLeftBracket = startLine != -1 && endLine == -1;         // if no right bracket then move left bracket, otherwise move right bracket
        boolean erased = false;
        if (getIndex(moveLeftBracket) == 0 && inc == -1) {      // if moving left will go up to previous line.
            if (getLine(moveLeftBracket) == 0) {
                return;                         // there is no previous line
            } else {
                drawBracket(g, moveLeftBracket);// remove bracket
                erased = true;                  // so it will not be removed again
                incLine(moveLeftBracket, -1);   // previous line
                int len = lines.get(getLine(moveLeftBracket)).length();
                setIndex(moveLeftBracket, len); // will be decremented by inc below
            }
        } else if (inc == 1 && getIndex(moveLeftBracket) == lines.get(getLine(moveLeftBracket)).length() - 1) {  // if moving right will go down to next line
            if (getLine(moveLeftBracket) == lines.size() - 1) {
                return;                         // there is no next line
            } else {    // move to next line
                drawBracket(g, moveLeftBracket);// remove bracket
                erased = true;                  // so it will not be removed again
                incLine(moveLeftBracket, 1);
                setIndex(moveLeftBracket, -1);  // will be incremented below
            }
        }
        if (!erased) {
            drawBracket(g, moveLeftBracket);     // XOR mode will erase it
        }
        incIndex(moveLeftBracket, inc);
        int temp = -1;

        // ignore zero-length characters (e.g., virama)
        do {
            temp = getPixelOffset(moveLeftBracket);
            setPixelOffset(moveLeftBracket, getPixelOffsetFromIndex(g, getLine(moveLeftBracket), getIndex(moveLeftBracket)));
            if (getPixelOffset(moveLeftBracket) == temp) {
                incIndex(moveLeftBracket, inc);
            }
        } while (getPixelOffset(moveLeftBracket) == temp);

        drawBracket(g, moveLeftBracket);     // redraw left bracket in new position
    }

    public void moveVertically(Graphics g, int kc) {
        int inc = 0;
        if (kc == KeyEvent.VK_UP) {
            inc = -1;
        } else if (kc == KeyEvent.VK_DOWN) {
            inc = 1;
        }
        boolean moveLeftBracket = startLine != -1 && endLine == -1;         // if no right bracket then move left bracket, otherwise move right bracket

        if ((getLine(moveLeftBracket) == 0 && inc == -1) || (getLine(moveLeftBracket) == lines.size() - 1 && inc == 1)) {
            return;                         // there is no previous or next line
        } else {
            drawBracket(g, moveLeftBracket);  // remove bracket
            incLine(moveLeftBracket, inc);    // destination line
            int len = lines.get(getLine(moveLeftBracket)).length();
            if (getIndex(moveLeftBracket) >= len) {
                setIndex(moveLeftBracket, len - 1);					// target line is shorter than current line so put bracket at end of target line.
                setPixelOffset(moveLeftBracket, this.getPixelOffsetFromIndex(g, getLine(moveLeftBracket), getIndex(moveLeftBracket)));
            } else {  // put bracket at approximately the same pixel position that it occupies now.
                setIndex(moveLeftBracket, getNearestCharToPixel(g, getLine(moveLeftBracket), getPixelOffset(moveLeftBracket)));
            }

            if (getLine(moveLeftBracket) < currentTopLine) {   // need to scroll down to see bracket
                currentTopLine = getLine(moveLeftBracket);
                repaint();
            } else if (getLine(moveLeftBracket) == currentTopLine + numLinesInPage) { // need to scroll up to see bracket
                currentTopLine = currentTopLine + 1;
                repaint();
            } else {
                drawBracket(g, moveLeftBracket);
            }
        }
    }

    public void drawBracket(Graphics g, boolean drawLeft) {
        if (drawLeft) {
            drawLeftBracket(g);             // remove bracket
        } else {
            //drawRightBracket(g);
            highlightSelection(g, startLine, startCharIndex, endLine, endCharIndex);
        }
    }

    public void highlight (TextPoint start, TextPoint end) {
        this.startLine = start.row;
        startCharIndex = start.col;
        this.endLine = end.row;
        endCharIndex = end.col;
        makeLineViewable(this.startLine, true);
        setCurrentTopLine(currentTopLine);
        Graphics g = _getGraphics();
        highlightSelection(g, startLine, startCharIndex, endLine, endCharIndex);
        _show(g);
    }
    
    public void makeLineViewable (int line, boolean middle) {
        if (middle) {
            currentTopLine = line - numLinesInPage/2;    // make currentTopLine numLinesInPage/2 positions above the line to center
            if (currentTopLine < 0) {
                currentTopLine = 0;
            }
        } else {
            currentTopLine = line;          // make line the first visible line
        }
    }

    public void keyReleased(KeyEvent e) {
    }

    public void setAllowMouseClick (boolean allowMouseClick) {
        this.allowMouseClick = allowMouseClick;
        requestFocusInWindow();
    }

    public void focusGained(FocusEvent e) {
        if (allowMouseClick) {
            justGotFocus = false;
            allowMouseClick = false;
        } else {
            justGotFocus = true;
        }
    }

    public void focusLost(FocusEvent e) {
    }

    public TextPoint getSelectionStart () {
        return new TextPoint(startLine, startCharIndex);
    }

    public TextPoint getSelectionEnd () {
        return new TextPoint(endLine, endCharIndex);
    }

        public TextPoint getTopLine () {
        return new TextPoint(currentTopLine, 0);
    }

    class CharInfo {

        int pixelOffset;
        int charIndex;
        int charWidth;

        public CharInfo(int pixelOffset, int charIndex, int charWidth) {
            this.pixelOffset = pixelOffset;
            this.charIndex = charIndex;
            this.charWidth = charWidth;
        }
    }
}
