package sita;

import org.sanskritlibrary.catalogs.ApplicationState;
import java.awt.Dimension;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.sanskritlibrary.CatalogRecord;
import org.sanskritlibrary.Regex;

public class ChooseTextDialog extends JDialog implements ListSelectionListener, ActionListener, WindowListener
{
    @Override
    public synchronized void addWindowListener(WindowListener l)
    {
        super.addWindowListener(l);
    }


    @Override
    public synchronized void addComponentListener(ComponentListener l)
    {
        super.addComponentListener(l);
    }

    int fileStartIndex = -1;
    boolean listConstructionInProgress = false;
    ArrayList<String> textTypes;
    ArrayList<CatalogRecord> directories;
    ArrayList<String> files;

    ApplicationState appstate;
    CatalogRecord result;
    JList textTypesList;
    DefaultListModel textTypesModel;
    JScrollPane textTypesScrollPane;

    JList directoryNamesList;
    DefaultListModel directoryNamesModel;
    JScrollPane directoryNamesScrollPane;

    JList fileNamesList;
    DefaultListModel fileNamesModel;
    JScrollPane fileNamesScrollPane;

    JList focusComponent = null;

    JLabel textInfo;

    JButton okayButton;
    JButton cancelButton;

    String error;

    boolean okClicked = false;

    public ChooseTextDialog(ApplicationState appstate)
    {
        this.addWindowListener(this);
        Utils.setEscapeKeyListener(this);
        this.appstate = appstate;
        Dimension listBoxSize = new Dimension(250, 500);
        setTitle("Choose text");
        setModal(true);
        setBounds(0, 0, 900, 600);
        GridBagLayout layout = new GridBagLayout();
        setLayout(layout);

        GridBagConstraints c = new GridBagConstraints();

// Row 1 Add labels
        JLabel textTypesLabel = new JLabel("Work archetype");
        c.gridy = 0;    // first row
        c.gridx = 0;    // first column
        add(textTypesLabel, c);

        JLabel directoryNamesLabel = new JLabel("Image file directories");
        c.gridy = 0;    // first row
        c.gridx = 1;    // second column
        add(directoryNamesLabel, c);

        JLabel fileNamesLabel = new JLabel("Page numbers");
        c.gridy = 0;    // first row
        c.gridx = 2;    // third column
        add(fileNamesLabel, c);

// Row 2 Add list boxes
        listConstructionInProgress = true;      // to prevent valueChanged event handler from being called when selectedIndex is initialized.
        // text names (e.g. Bhagavad Gita)
        textTypesModel = new DefaultListModel();
        textTypes = appstate.getManuscriptTypes();
        int n = 0;
        for (int i=0; i<textTypes.size(); i++)
        {
            String textType = textTypes.get(i);
            if (appstate.getDirectories(textType).size() > 0)
                textTypesModel.add(n++, textType);
        }
        textTypesList = new JList(textTypesModel);
        textTypesList.addListSelectionListener(this);
        textTypesList.setVisibleRowCount(25);
        textTypesScrollPane = new JScrollPane(textTypesList);
        textTypesScrollPane.setPreferredSize(listBoxSize);
        textTypesScrollPane.setMinimumSize(new Dimension(250, 100));

        c.gridy = 1;    // second row
        c.gridx = 0;    // first column

        add(textTypesScrollPane, c);

        // manuscript directories containing the selected text.
        directoryNamesModel = new DefaultListModel();
        directoryNamesList = new JList(directoryNamesModel);
        directoryNamesList.addListSelectionListener(this);
        directoryNamesList.setVisibleRowCount(25);
        directoryNamesScrollPane = new JScrollPane(directoryNamesList);
        directoryNamesScrollPane.setPreferredSize(listBoxSize);
        directoryNamesScrollPane.setMinimumSize(new Dimension(250, 100));
        c.gridy = 1;    // second row
        c.gridx = 1;    // second column

        add(directoryNamesScrollPane, c);

        // pages in selected manuscript directory.
        fileNamesModel = new DefaultListModel();
        fileNamesList = new JList(fileNamesModel);
        fileNamesList.addListSelectionListener(this);
        fileNamesList.setVisibleRowCount(25);
        fileNamesScrollPane = new JScrollPane(fileNamesList);
//        fileNamesScrollPane.setSize(listBoxSize);
        fileNamesScrollPane.setPreferredSize(listBoxSize);
        fileNamesScrollPane.setMinimumSize(new Dimension(250, 100));
        c.gridy = 1;    // second row
        c.gridx = 2;    // third column

        add(fileNamesScrollPane, c);

// Row 3 Information area
        textInfo = new JLabel("Info");
        c.gridy = 2;        // third row
        c.gridx = 0;        // first column
        c.gridwidth = 3;    // three columns wide

        add(textInfo, c);

        // now initialize lists to be what they were when the user stopped last time.
        int idx = appstate.getLastTextIdx();
        if (idx != -1)
        {
            if (textTypesList.getModel().getSize() <= idx) {
                idx = 0;
            }
            textTypesList.setSelectedIndex(idx);
            appstate.setCurrentTextType((String)textTypesList.getSelectedValue());
        }

        idx = appstate.getLastManuscriptIdx();
        if (idx != -1)
        {
            directories = appstate.getDirectories((String)textTypesList.getSelectedValue());
            populateDirectoryNames();
            if (directoryNamesList.getModel().getSize() <= idx) {
                idx = 0;
            }

            populateFileNames(idx);
            directoryNamesList.setSelectedIndex(idx);
        }

        idx = appstate.getLastPageIdx();
        if (idx != -1)
        {
            if (fileNamesList.getModel().getSize() <= idx) {
                idx = 0;
            }
            fileNamesList.setSelectedIndex(idx);
            appstate.setCurrentFileName(files.get(idx));
            textInfo.setText(getFileName());
        }

        listConstructionInProgress = false;         // handle list events again

// Row 4 Buttons
        okayButton = new JButton("OK");
        okayButton.addActionListener(this);
        okayButton.setActionCommand("ok");

        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(this);
        cancelButton.setActionCommand("cancel");

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(okayButton);
        buttonPanel.add(cancelButton);
        c.gridy = 3;        // fourth row
        c.gridx = 0;        // first column
        c.gridwidth = 3;    // three columns wide

        add(buttonPanel, c);

        getRootPane().setDefaultButton(okayButton);

        Utils.centerScreen(this);
    }

    public void display()
    {
        if (appstate.getLastPageIdx() != -1)
        	focusComponent = fileNamesList;
        else if (appstate.getLastManuscriptIdx() != -1)
        	focusComponent = directoryNamesList;
        else
        	focusComponent = textTypesList;

        java.awt.EventQueue.invokeLater(new Runnable()
        {
			public void run()
            {
				 focusComponent.grabFocus();
			}
		});

        setVisible(true);
    }

    public String getChosenText()
    {
        return getFileName();
    }

    public String getProgress()
    {
        int n = directoryNamesList.getSelectedIndex();
        if (n >= 0 && n < directories.size())
        {
            CatalogRecord cr = directories.get(n);
            String annotated = "";
            int numAnnotations = appstate.getNumAnnotations();
            if (numAnnotations == 0)
                annotated = String.format("(no annotations)");
            else if (numAnnotations == 1)
                annotated = String.format("(%d annotation)", numAnnotations);
            else if (numAnnotations > 1)
            {
                int num = appstate.getNumberOfCurrentAnnotation();
                if (num != 0)
                    annotated = String.format("(%d of %d annotations)", num, numAnnotations);
                else
                    annotated = String.format("(%d annotations)", numAnnotations);
            }

            String usingCDATA = "";
            if (!appstate.getUseCDATA())
                 usingCDATA = " [**not** using CDATA]";
            String progress = String.format("page name %s of %d files %s %s",
                                            appstate.getLastPage(),
                                            cr.numManuscriptFiles,
                                            annotated, usingCDATA);

            return progress;
        }
        else
            return "";
    }

    public String getNextText()
    {
        int idx = fileNamesList.getSelectedIndex();
        if (idx != fileNamesModel.getSize() - 1)
        {
            okClicked = true;       // simulate user clicking the OK button
            fileNamesList.setSelectedIndex(idx+1);
            return getFileName();
        }
        else
            return null;
    }

    public String getPrevText()
    {
        int idx = fileNamesList.getSelectedIndex();
        if (idx != 0)
        {
            okClicked = true;       // simulate user clicking the OK button
            fileNamesList.setSelectedIndex(idx-1);
            return getFileName();
        }
        else
            return null;
    }

    public void valueChanged(ListSelectionEvent e)
    {
        if (listConstructionInProgress)
            return;

        JList src = (JList)e.getSource();
        if (src == textTypesList)
        {
            listConstructionInProgress = true;
            String text = (String)src.getSelectedValue();
            appstate.setCurrentTextType(text);
            directories = appstate.getDirectories(text);
            directoryNamesModel.clear();
            fileNamesModel.clear();
            populateDirectoryNames();
            listConstructionInProgress = false;
        }
        else if (src == directoryNamesList)
        {
            listConstructionInProgress = true;
            int idx = src.getSelectedIndex();
            populateFileNames(idx);
            listConstructionInProgress = false;
        }
        else if (src == fileNamesList)
        {
            int fileidx = fileNamesList.getSelectedIndex();
            appstate.setCurrentFileName(files.get(fileidx));
            textInfo.setText(getFileName());
        }
    }


    private void populateDirectoryNames()
    {
        for (int i = 0; i < directories.size(); i++)
        {
            directoryNamesModel.add(i, directories.get(i).directoryName);
        }
        directoryNamesList.setVisible(true);

    }

    private void populateFileNames(int idx)
    {
        Utils.displayWaitCursor(this);
        CatalogRecord cr = directories.get(idx);
        appstate.setCurrentCatalogRecord(cr);
        try
        {
            files = cr.getPages();
        } catch (Exception ex)
        {
            Logger.writeLog(ex);
            error = ex.getMessage();
        }
        fileNamesModel.clear();

        fileStartIndex = -1;
        for (int i = 0, j = 0; i < files.size(); i++) {
            String fn = files.get(i);
            if (fileStartIndex == -1) {
                fileStartIndex = i;
            }

            Regex re = new Regex(fn, "body(.*)\\.jpg");
            if (re.find()) {
                fileNamesModel.add(j, re.getGroup(1));
            } else {
                fileNamesModel.add(j, fn);
            }
            j++;
        }
        fileNamesList.setVisible(true);
        Utils.displayDefaultCursor(this);
    }


    public void setCurrentPage(String manuscriptName, String pageName)
    {
        int len = directoryNamesModel.size();
        for (int i = 0; i < len; i++)
        {
            if (manuscriptName.equals(directoryNamesModel.get(i)))
            {
                if (i != appstate.getLastManuscriptIdx())
                {
                    directoryNamesList.setSelectedIndex(i);         // so that fileNames model gets populated
                    break;
                }
            }
        }

        len = fileNamesModel.size();
        for (int i = 0; i < len; i++)
        {
            if (pageName.equals(fileNamesModel.getElementAt(i)))
            {
                fileNamesList.setSelectedIndex(i);
                appstate.setCurrentFileName(files.get(i));
                textInfo.setText(getFileName());
                break;
            }
        }

        return;
    }

    private String getFileName()
    {
        if (!okClicked)
            return null;
        
        int diridx = directoryNamesList.getSelectedIndex();
        if (diridx == -1)
            return null;

        int fileidx = fileNamesList.getSelectedIndex();
        if (fileidx == -1)
            return null;

        appstate.setLastTextIdx(textTypesList.getSelectedIndex());
        appstate.setLastText((String)textTypesList.getSelectedValue());

        appstate.setLastManuscriptIdx(diridx);
        appstate.setLastManuscript((String)directoryNamesList.getSelectedValue());

        appstate.setLastPageIdx(fileidx);
        appstate.setLastPage((String)fileNamesList.getSelectedValue());

        StringBuilder sb = new StringBuilder();

        sb.append(appstate.getManuscriptRootDirectory());
        sb.append("/");
        sb.append(directories.get(diridx).directoryName);
        sb.append("/");
        //sb.append(getPathInfo((String)fileNamesList.getSelectedValue()).filename);
        int idx = fileNamesList.getSelectedIndex();
        sb.append(files.get(idx + fileStartIndex));
        return sb.toString();
    }

// TODO
    String getPathInfo(String filename)       // get the real file name
    {
        for (String file: files)
        {
            if (file.contains(filename))
                return file;
        }

        return null;
    }


    public void actionPerformed(ActionEvent e)
    {
        okClicked = e.getActionCommand().equals("ok");
        if (okClicked)
        {
            if (fileNamesList.getSelectedIndex() == -1)
            {
                Utils.displayWarning("Please select a page");
                return;
            }
        }
            
        setVisible(false);          // to do, if cancel, restore to previous values
    }

    public void windowOpened(WindowEvent e)
    {
    }

    public void windowClosing(WindowEvent e)
    {
        okClicked = false;
    }

    public void windowClosed(WindowEvent e)
    {
    }

    public void windowIconified(WindowEvent e)
    {
    }

    public void windowDeiconified(WindowEvent e)
    {
    }

    public void windowActivated(WindowEvent e)
    {
    }

    public void windowDeactivated(WindowEvent e)
    {
    }
}
