package sita;

import org.sanskritlibrary.catalogs.RawComment;
import org.sanskritlibrary.catalogs.CommentInfo;
import org.sanskritlibrary.catalogs.ApplicationState;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.DefaultListModel;
import javax.swing.InputMap;
import javax.swing.JDialog;
import javax.swing.JLabel;

import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.sanskritlibrary.Regex;

public class CommentsDialog extends JDialog implements ListSelectionListener
{
    boolean ignoreValueChanged = false;
    boolean useCurrentManuscript = true;

    Component _this;

    ArrayList<RawComment> foundComments = new ArrayList<RawComment>();
    JList resultsList;
    DefaultListModel resultsListModel;
    JScrollPane resultsListScrollPane;

    JList tagsList;
    DefaultListModel tagsListModel;
    JScrollPane tagsListScrollPane;

    JLabel messageLabel;
    JTextField query;

    ApplicationState appstate;
    ArrayList<CommentInfo> commentTags;
    ArrayList<Integer>mapTags = new ArrayList<Integer>();

    int currentTagNumber = -1;

    public CommentsDialog(ApplicationState appstate)
    {
        this.appstate = appstate;

        setLayout(new BorderLayout());

        JPanel topPanel = new JPanel(new BorderLayout());

        tagsListModel = new DefaultListModel();
        tagsList = new JList(tagsListModel);
        tagsList.addListSelectionListener(this);
        tagsList.setVisibleRowCount(10);
        tagsListScrollPane = new JScrollPane(tagsList);
        tagsListScrollPane.setPreferredSize(new Dimension(140, 200));
        tagsListScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        commentTags = appstate.getCommentTags();
        int len = commentTags.size();
        int j = 0;
        for (int i=0; i<len; i++)
        {
            if(appstate.commentsSummary.get(i).size() > 0)
            {
                mapTags.add(i);     // map j -> i
                tagsListModel.add(j++, commentTags.get(i).tag);
            }
        }

        resultsListModel = new DefaultListModel();
        resultsList = new JList(resultsListModel);
        resultsList.addListSelectionListener(this);
        resultsList.setVisibleRowCount(10);
        resultsListScrollPane = new JScrollPane(resultsList);
        resultsListScrollPane.setPreferredSize(new Dimension(140, 200));
        resultsListScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        topPanel.add(resultsListScrollPane, BorderLayout.CENTER);
        topPanel.add(tagsListScrollPane, BorderLayout.EAST);

        add(topPanel, BorderLayout.CENTER);

        JPanel bottomPanel = new JPanel(new BorderLayout());

        query = new JTextField();
        query.setPreferredSize(new Dimension(300, 30));

        InputMap im = query.getInputMap();
        ActionMap am = query.getActionMap();
        im.put(KeyStroke.getKeyStroke("ENTER"), "search");
        am.put("search", new SearchAction());
        appstate.getAnnotater().setupInputActions(im, am);

        bottomPanel.add(query, BorderLayout.CENTER);

        messageLabel = new JLabel();
        messageLabel.setPreferredSize(new Dimension(300, 40));
        bottomPanel.add(messageLabel, BorderLayout.SOUTH);
        add(bottomPanel, BorderLayout.SOUTH);

        setBounds(0, 0, 350, 250);
        Utils.upperRightScreen(this);
        setModal(false);
        setAlwaysOnTop(true);

        _this = this;               // so that SearchAction can access object.
    }

    public void display(boolean useCurrentManuscript)
    {
        ignoreValueChanged = true;
        this.useCurrentManuscript = useCurrentManuscript;
        setVisible(true);
        tagsList.clearSelection();
        resultsListModel.clear();
        resultsList.clearSelection();
        query.setText("");
        messageLabel.setText("");
        ignoreValueChanged = false;
    }

    public void valueChanged(ListSelectionEvent e)
    {
        if (ignoreValueChanged)
            return;

        JList src = (JList)e.getSource();
        if (src == resultsList)
            displaySelectedImage();
        else if (src == tagsList)
            updatePages(tagsList.getSelectedIndex());
    }

    private void displaySelectedImage()
    {
        Utils.displayWaitCursor(this);
        ignoreValueChanged = true;
        appstate.getAnnotater().saveState();
        try
        {
            String rootDirectory = appstate.getManuscriptRootDirectory();
            String manuscriptDirectory = appstate.getLastManuscript();
            String lastPage = (String) resultsList.getSelectedValue();
            String pageName = appstate.getImageFilename(manuscriptDirectory, lastPage + ".jpg");
            String imagefile = String.format("%s/%s/%s", rootDirectory, manuscriptDirectory, pageName);

            appstate.getAnnotater().setCurrentPage(manuscriptDirectory, lastPage);
            int n = 0;
            if (currentTagNumber != -1)
            {
                Hashtable<String, String> h = appstate.commentsSummary.get(currentTagNumber);
                String info = h.get(lastPage);
                messageLabel.setText(info);

                Regex re = new Regex(info, "(\\d+)");
                if (re.find())
                    n = Integer.parseInt(re.getGroup(1));
            }
            else
                n = foundComments.get(resultsList.getSelectedIndex()).annotationNumber + 1;
            appstate.setMoveToAnnotationNumber(n - 1);
            appstate.getAnnotater().displayManuscript(imagefile);
            appstate.getAnnotater().bringToFront();
        } catch (Exception ex)
        {
            Utils.displayError(ex);
        }
        ignoreValueChanged = false;
        Utils.displayDefaultCursor(this);
    }


    private void updatePages(int n)
    {
        ignoreValueChanged = true;

        currentTagNumber = mapTags.get(n);

        Hashtable<String, String> h = appstate.commentsSummary.get(currentTagNumber);
        ArrayList<String> keys = new ArrayList<String>();
        Enumeration<String> en = h.keys();
        while (en.hasMoreElements())
        {
            keys.add(en.nextElement());
        }

        Collections.sort(keys);

        resultsListModel.clear();
        int i = 0;
        for (String key : keys)
        {
            resultsListModel.add(i, key);
            i++;
        }
        messageLabel.setText("");
        ignoreValueChanged = false;
    }

    private class SearchAction extends AbstractAction
    {
        public void actionPerformed(ActionEvent ev)
        {
            ignoreValueChanged = true;
            currentTagNumber = -1;
            String text = query.getText().toLowerCase();

            if (text.length() == 0)
                return;

            foundComments.clear();
            resultsListModel.clear();
            int n = 0;
            for (RawComment rc : appstate.commentsList)
            {
                if (rc.description.contains(text))
                {
                    resultsListModel.add(n, rc.filekey);
                    foundComments.add(rc);
                    n++;
                }
            }

            if (n == 0)
                JOptionPane.showMessageDialog(_this, "Not found");
            else
            {
                query.setText("");
                messageLabel.setText("");
                tagsList.clearSelection();
            }
            ignoreValueChanged = false;
        }
    }

}
