package sita;

import org.sanskritlibrary.catalogs.ApplicationState;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.JDialog;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import java.awt.Dimension;
import javax.swing.JPanel;

public class ConfigurationDialog extends JDialog implements ListSelectionListener, ActionListener
{
    String manuscriptRootDirectory = null;
    String catalogDirectory = null;
    String textDirectory = null;
    String annotationsBackupDirectory =  null;

    GridBagConstraints c = new GridBagConstraints();
    JLabel manuscriptRootDirectoryLabel;
    JLabel digitizedTextDirectoryLabel;
    JLabel catalogDirectoryLabel;
    JLabel annotationsBackupDirectoryLabel;
    ApplicationState appstate;
    JButton okayButton;
    JButton cancelButton;

    String error;
    
    public ConfigurationDialog(ApplicationState appstate)
    {
        this.appstate = appstate;
        Utils.setEscapeKeyListener(this);
        setLayout(new BorderLayout());
        add(createButtonPanel(), BorderLayout.NORTH);
        add(createTextPanel(), BorderLayout.CENTER);
        add(createButtonPanel02(), BorderLayout.SOUTH);
        setModal(true);
        setBounds(0, 0, 800, 250);
        setTitle("Configuration");
        Utils.centerScreen(this);
    }


    private JPanel createTextPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        

        return panel;
    }


    private JPanel createButtonPanel02()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        GridBagConstraints cs = new GridBagConstraints();

        okayButton = new JButton("OK");
        okayButton.addActionListener(this);
        okayButton.setActionCommand("okay");
        cs.gridx = 0;
        cs.gridy = 0;
        panel.add(okayButton, cs);

        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(this);
        cancelButton.setActionCommand("cancel");
        cs.gridx = 1;

        getRootPane().setDefaultButton(okayButton);

        panel.add(cancelButton, cs);
        return panel;

    }


    private JPanel createButtonPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());

        manuscriptRootDirectoryLabel = addButton(panel, 0, "Manuscript root directory", appstate.getManuscriptRootDirectory(), "manuscripts");

        digitizedTextDirectoryLabel = addButton(panel, 1, "Digital text directory", appstate.getTextDirectory(), "texts");

        catalogDirectoryLabel = addButton(panel, 2, "Catalog directory", appstate.getCatalogDirectory(), "catalog");

        annotationsBackupDirectoryLabel = addButton(panel, 3, "Backup directory", appstate.getAnnotationsBackupDirectory(), "backup");

        return panel;
    }

    private JLabel addButton(JPanel panel, int row, String type, String value, String actionCommand)
    {
        JLabel lblType = new JLabel(type);
        lblType.setHorizontalAlignment(SwingConstants.LEFT);

        JLabel lblValue = new JLabel(value);
        lblValue.setHorizontalAlignment(SwingConstants.LEFT);
        lblValue.setMinimumSize(new Dimension(400, 20));

        JButton b = new JButton("Change");
//        b.setSize(new Dimension(75, 15));
        b.setMinimumSize(new Dimension(95, 20));
        b.addActionListener(this);
        b.setActionCommand(actionCommand);

        c.gridy = row;
        c.gridx = 0;
        c.fill = GridBagConstraints.NONE;
        c.ipadx = 20;
        c.ipady = 20;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.weightx = 0.0;
        panel.add(lblType, c);              // name of directory

        c.gridx = 1;
        panel.add(lblValue, c);             // value of directory

        c.gridx = 2;
        c.ipadx = 0;
        c.ipady = 0;
        panel.add(b, c);                    // button

        return lblValue;
    }
    
    public void valueChanged(ListSelectionEvent e)
    {
    }

    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand().equals("okay"))
        {
            try
            {
                if (manuscriptRootDirectory != null)
                    appstate.setManuscriptRootDirectory(manuscriptRootDirectory);
                if (catalogDirectory != null)
                    appstate.setCatalogDirectory(catalogDirectory);
                if (textDirectory != null)
                    appstate.setTextDirectory(textDirectory);
                if (annotationsBackupDirectory != null) {
                    appstate.setAnnotationsBackupDirectory(annotationsBackupDirectory);
                }
                appstate.update();
                setVisible(false);
            }
            catch (Exception ex)
            {
                Utils.displayError(ex);
            }
        }
        else if (e.getActionCommand().equals("cancel"))
        {
            setVisible(false);
        }
        else
        {
            String currentDirectory = null;
            if (e.getActionCommand().equals("manuscripts")) {
                currentDirectory = appstate.getManuscriptRootDirectory();
            } else if (e.getActionCommand().equals("texts")) {
                currentDirectory = appstate.getTextDirectory();
            } else if (e.getActionCommand().equals("catalog")) {
                currentDirectory = appstate.getCatalogDirectory();
            } else if (e.getActionCommand().equals("backup")) {
                currentDirectory = appstate.getAnnotationsBackupDirectory();
            }
            JFileChooser chooser = new JFileChooser();
            chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            chooser.setCurrentDirectory(new File(currentDirectory));
            int returnVal = chooser.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION)
            {
                String filename = chooser.getSelectedFile().getAbsolutePath();

                try
                {
                    if (e.getActionCommand().equals("manuscripts"))
                    {
                        manuscriptRootDirectory = filename;
                        manuscriptRootDirectoryLabel.setText(filename);
                    }
                    else if (e.getActionCommand().equals("texts"))
                    {
                        textDirectory = filename;
                        digitizedTextDirectoryLabel.setText(filename);
                    }
                    else if (e.getActionCommand().equals("catalog"))
                    {
                        catalogDirectory = filename;
                        catalogDirectoryLabel.setText(filename);
                    }
                    else if (e.getActionCommand().equals("backup"))
                    {
                        annotationsBackupDirectory = filename;
                        annotationsBackupDirectoryLabel.setText(filename);
                    }
                } catch (Exception ex)
                {
                    Utils.displayError(ex);
                }
            }
        }
    }

    public String getError()
    {
        return error;
    }

}
