package sita;

/*
Hi Peter,

I have fixed the scrolling issues on the Mac and am now looking carefully at the behavior of the TextArea.

My original idea for the TextArea was that it would behave exactly as the Manuscript area, that is, the first click would mark the beginning character of the text annotation and the second click would mark the end character. Any clicks after the second click would reposition the end character and reclicking on the start character would remove the annotation. You pointed out that you would like the text between the start and end characters highlighted and I have done this.

However, besides the highlighting there is still a difference in the behavior the Manuscript area and the TextArea. The Manuscript area is always "live", anytime you click on it you will modify the annotation somehow. However, the TextArea does not respond to the first click when it regains focus. Why haven't you complained about the "liveness" of the Manuscript area? Is it because you don't mind using the scroll bar on the Manuscript area but would prefer to use the up and down arrows in the TextArea? Does this have something to do with the content of the TextArea being so much larger than the jpeg image displayed in the Manuscript area?

Anyway, here is the current design of the TextArea.

1. If it does not have focus, clicking on it gives it the focus and does not change the annotation.
2. When the TextArea has focus there will be a thin red line in the left margin of the TextArea.
3. If there is no start character, the next click marks the start character of the annotation
4. The second and subsequent clicks mark the end character of the annotation, highlighting the text between the start character and the end character.
5. Shift + an arrow key repositions the end character of the annotation, modifying the highlight appropriately. If the end character hasn't been selected yet, the Shift+arrow key will move the start character. If there is no start character, Shift + arrow key has no effect.
6. An arrow key without a shift will scroll the text in the TextArea.
7. Esc will display the current annotation (so you can scroll and return to the annotation)
8. Del will delete the annotation
9. I have not implemented a cursor(caret) for the TextArea. The only time this is a problem is when you create a new annotation and would like some indication of where the old annotation ended. Instead of a cursor, I automatically highlight the character after the end character of the previous annotation (skipping whitespace). This is my guess of what the start character of the next annotation will be. If this is not correct, then you can click it to delete it or use the del key.
10. Searching works almost as before. You type the regular expression to search for into the search area and hit enter. This will highlight and display the first match. Hitting Enter will display the next match. Shift+Enter will display the previous match. Since there is no cursor now, the search begins at the first visible row in the TextArea.


Please let me know if you want to modify this behavior in some way.

thanks,
--ralph

 */

import org.sanskritlibrary.catalogs.SegmentInfo;
import org.sanskritlibrary.catalogs.OffsetInfo;
import org.sanskritlibrary.catalogs.ApplicationState;
import org.sanskritlibrary.catalogs.TextMap;
import org.sanskritlibrary.catalogs.TextPoint;
import org.sanskritlibrary.catalogs.AnnotationInfo;
import java.util.Hashtable;
import java.awt.Font;
import java.awt.Point;
import java.io.File;
import javax.swing.JScrollPane;
import org.sanskritlibrary.webservice.TransformMap;
import org.sanskritlibrary.webservice.WebServices;
import org.sanskritlibrary.ResourceInputStream;
import org.sanskritlibrary.ResourceInputStreamImpPath;
import org.sanskritlibrary.FSM;
import org.sanskritlibrary.Regex;
import java.util.ArrayList;
import javax.swing.SwingUtilities;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.TabInfo;
import org.sanskritlibrary.Text;

public final class DigitalTextPage extends AnnotationPanel
{
    String currentDigitalText = null;
    int currentFontsize = 0;
    Hashtable<String, FSM[]> fsmh = new Hashtable<String, FSM[]>();
    ResourceInputStream ris = new ResourceInputStreamImpPath("transcoders");
    JScrollPane scroller;
    String error = "";                                  // contains error message if image load fails
    protected String lastSearchedText;                  // to detect if use changed search query
    protected int lastMatchedLine = 0;
    protected int startAnnotation = 0;                  // set to where user clicked mouse. Use to let user move arrow keys and then hit enter to highlight covered text
    private ArrayList<TextMap> textmap;
    private TextPoint guessNext = null;

    public DigitalTextPage(ApplicationState appstate, int rows, int cols, TextSearch query)
    {
        super(appstate, 0, 0, rows, cols);
        //appstate.getAnnotater().setupInputActions(im, am);
    }

    public void search(String text, int requestType) throws Exception
    {
        /*
           requestType
         *    0 Normal search, highlight next found string
         *    1 Go back to previous found string
         */
        if (lastSearchedText == null || !text.equals(lastSearchedText))     // if this is a new query
        {
            requestType = 0;
            ac.hits.clear();
            lastSearchedText = text;
            TextPoint lastSearchedOffset = ac.getSelectionStart();
            if (!lastSearchedOffset.exists()) {
                lastSearchedOffset = ac.getTopLine();
            }

            ac.removeMatchHighlights();

            String encoding = appstate.getEncoding();                   // current encoding of digital text
            if (appstate.getSearchWithSLP1() && !encoding.equals("slp1"))    // convert to SLP1 if appropriate
            {
                String map = String.format("*:slp1->%s", appstate.getEncoding());
                TransformMap tm = new TransformMap(map, ris, fsmh);
                // Replace regular expression metacharacters so that they don't get converted
                text = text.replaceAll("\\\\", "\u0001");
                text = text.replaceAll("\\[", "\u0002");
                text = text.replaceAll("\\]", "\u0003");

                text = WebServices.transformString(text, tm);

                // Put metacharacters back.
                text = text.replaceAll("\\[\u0001\\]", "\\\\");            // put escape back
                text = Utils.removeBraces(text, '[');
                text = text.replaceAll("\u0002", "[");
                text = text.replaceAll("\u0003", "]");
            }

            // find all strings that match the query
            Regex re = new Regex("", text);
            int numLines = ac.getNumLines();
            for (int i=0; i<numLines; i++) {
                String line = ac.getLineText(i);
                re.reset(line);
                if (re.find())          // if query was found in text
                {
                    int idx = re.getOffset(0);     // offset of found text
                    ac.hits.add(new TextSelection(idx, idx + re.getGroup(0).length() - 1,  i));           // remember start of all found words in cause user hits
                }
            }
            if (ac.hits.size() == 0) {
                Utils.displayWarning("Not found");          // query string not found in text
                return;
            }
            ac.currentHit = -1;
        }

        if (requestType == 0) {
            if (ac.currentHit == ac.hits.size() - 1) {
                Utils.displayWarning("At last match");
            } else {
                ac.currentHit++;
                TextSelection ts = ac.hits.get(ac.currentHit);
                ac.showLine(ts);
                setScroller(ac.getTopLine());
            }
        }
        else if (requestType == 1)      // control Enter, going backwards
        {
            if (ac.currentHit == 0) {
                Utils.displayWarning("At first match.");
            } else {
                ac.currentHit--;
                ac.showLine(ac.hits.get(ac.currentHit));
                setScroller(ac.getTopLine());
            }
        }
    }


    private TextPoint nextCharacter (TextPoint last) {
        return last; //TODO
    }

    private int getLineNumber(TextPoint last)
    {
        return last.row;
    }

    public void setScroller (int line) {
        this.setScroller(new TextPoint(line, 0));
    }

    public Point getSelected()
    {
        // TODO!
        return null;
    }


    public void loadDigitalText(int tabnum, String encoding) throws Exception {
        // TODO check if tabs already have desired texts
        appstate.setCurrentTabNumber(tabnum);
        textmap = new ArrayList<TextMap>();
        appstate.setTextMap(textmap);
        TabInfo ti = appstate.getCurrentTabInfo();

        currentFontsize = appstate.getDefaultFontsize();
        currentFontsize = 24;  //REB
        Font f = new Font("Praja", Font.TRUETYPE_FONT, currentFontsize);
        this.setPanelFont(f);

//        Utils.displayWaitCursor(this);
        int linenum = 0;
        ArrayList<String> lines = new ArrayList<String>();
        int startLine = 0;
        for (Text text : ti.texts) {
            TextMap tm = null;
            if (text.isVirtual) {
                startLine = linenum;    // This is where virtual file begins in tab
            } else {
                tm = new TextMap();     // if non-virtual only need 1 TextMap for this Text
                tm.tabStartrow = linenum;
            }

            String[] ls = null;         // this will hold lines of this Text
            if (text.type == Text.Type.Bookmark) {              // its a comment to be inserted into the tab panel
                ls = text.ref.split("\n");
                tm.type = Text.Type.Bookmark;       // Note: tm will be defined because if text is virtual then it will not be a bookmark! Rather any bookmarks will be defined in the SegmentInfo of the virtual file
            } else if (text.type == Text.Type.Comment) {
                ls = text.ref.split("\n");
                tm.type = Text.Type.Comment;        // Note: tm will be defined because if text is virtual then it will not be a comment! Rather any bookmarks will be defined in the SegmentInfo of the virtual file
            } else {                                // its a file to be inserted into the tab panel
                try {
                    String filename;
                    if (text.isVirtual) {
                        filename = String.format("%s/virtual/%s.%s", appstate.getTextDirectory(), text.ref, encoding);
                    } else {
//                        tm.filename = String.format("%s.%s", text.ref, encoding);
                        tm.filename = text.ref;
                        tm.type = Text.Type.TextRef;
                        filename = String.format("%s/%s.%s", appstate.getTextDirectory(), text.ref, encoding);  // filename will be used to get the SegmentInfo for this virtual file and these will be used to create the TextMaps
                    }

                    if (new File(filename).exists() == true) {   // filename is eithe a source file or a virtual file that was defined in config.xml and created by the File/Update Virtual Files command.
                        ls = IOAssistance.readEntireFile(filename).split("\n");
                    } else {        // either an error in catalog.xml or virtual file not created yet.
                        ls = new String[]{String.format("%s missing", filename)};
                    }
                } catch (Exception ex) {
                    throw ex;
                }
            }

            int fileLineCount = 0;
            for (String l : ls) {       // add lines of just read file to tab.
                lines.add(l);
                linenum++;
                fileLineCount++;
            }

            if (text.isVirtual) {       // get the offsets for parts of the virtual file (i.e. its bookmarks, comments and text
                ArrayList<SegmentInfo> sis = appstate.getVirtualFile(text.ref);
                ArrayList<OffsetInfo> offsets = appstate.getOffsets(text.ref);
                if (offsets == null) {      // not good. Need to do Files/Create Virtual Files
                    lines.clear();
                    lines.add("Need to create virtual files (Files/Create Virtual Files).");
                    break;
                }
                // Have the virtual file. Now create a TextMap for each of its SegmentInfos.
                int len = sis.size();
                for (int i=0; i<len; i++) {
                    SegmentInfo si = sis.get(i);
                    int last = 0;
                    if (i != len - 1) {
                        last = startLine+offsets.get(i+1).offsetInVirtualFile-1;    // last is row number of last line of this segment in the tab
                    } else {
                        last = linenum - 1;    // row number of last segment is the current last row of the tab.
                    }

                    // At this point we know the start and end rows of this segment in the tab.
                    // If this segment is text then we also need to save the start line of the original text so that we can compute annotation offsets later.

                    tm = new TextMap(si.getType(),
                            si.source,
                            startLine+offsets.get(i).offsetInVirtualFile,
                            last,
                            offsets.get(i).startLineInSourceFile,
                            offsets.get(i).endLineInSourceFile);
                    textmap.add(tm);
                }
            } else {            // text is a source text. There is nothing more to do with this Text.
                tm.tabEndrow = linenum - 1;
                tm.textStartrow = 0;
                tm.textEndrow = fileLineCount - 1;
                textmap.add(tm);
            }
        }

        setText(lines);
    }


    public void increaseFontSize()
    {
        Font f = getFont();
        int size = f.getSize() + 3;
        appstate.setFontSize(size);
        setPanelFont(new Font(f.getName(), f.getStyle(), size));
    }

    public void decreaseFontSize()
    {
        Font f = getFont();
        int size = f.getSize() - 3;
        appstate.setFontSize(size);
        setPanelFont(new Font(f.getName(), f.getStyle(), size));
    }

    public void clearAnnotation()
    {
        TextPoint tp = ac.getSelectionEnd();
        ac.removeSelection();
        if (tp.row != -1) {           // if -1 then we are here because a next or previous page was done.
            ac.markEndOfLastAnnotation(guessNext);      // use XOR to get rid of previous
            guessNext = guessNextStart(tp);       // guessNext is first non-whitespace character after end of this annotation.
            ac.markEndOfLastAnnotation(guessNext);
        }
    }

    public void clearEndOfPreviousAnnotation () {           // assume one is already there
        ac.markEndOfLastAnnotation(guessNext);
    }

    public TextPoint getEndOfPreviousAnnotation () {
        return guessNext;
    }

    public TextPoint getStartOfCurrentAnnotation () {
        return ac.getStartOfCurrentAnnotation();
    }

    public String getError()
    {
        return error;
    }


    public void display() {
        AnnotationInfo ai = appstate.getCurrentAnnotation();
        if (ai.hasSelectedText()) {
            ac.highlight(ai.getTextStart(textmap), ai.getTextEnd(textmap));
            vsb.setValue(ac.getTopLine().row);
        }
    }

    private TextPoint guessNextStart(TextPoint tp) {
        if (tp.row != -1) {
            int row = tp.row;
            int col = tp.col + 1;
            String s = ac.lines.get(row);
            int len = s.length();
            for (int c = col; c < len; c++) {
                if (Character.isWhitespace(s.charAt(c))) {
                        return new TextPoint(row, col);
                }
            }
            return new TextPoint(row, s.length());
        }

        return null;
    }

    public void saveSelection() {
        if (selectionExists()) {
            appstate.clearTextMarkers();        // in case this annotation was updated
            generateTextMarkers(getSelectionStart(), getSelectionEnd(), textmap, ac.lines);
        }
    }

    public void generateTextMarkers(TextPoint annotationStart, TextPoint annotationEnd,
            ArrayList<TextMap> textmap, ArrayList<String> lines) {
        int currentTMIndex = 0;
        int startRow = 0;
        int startCol = 0;
        int endRow = 0;
        int endCol = 0;
        int startIndex = -1;
        int endIndex = -1;

        // Find text that contains the start of the annotation
        while (startIndex == -1 && currentTMIndex < textmap.size()) {
            TextMap tm = textmap.get(currentTMIndex);
            if (annotationStart.row <= tm.tabEndrow) {
                if (tm.type == Text.Type.TextRef) {
                    startIndex = currentTMIndex;
                } else {
                    break;          // annotation starts in a comment or bookmark
                }
            } else {
                currentTMIndex++;
            }
        }

        int saveTMIndex = currentTMIndex;
        boolean adjusted = false;
        if (startIndex == -1) {      // annotation starts in a comment or bookmark, find next textref
            currentTMIndex++;
            while (startIndex == -1 && currentTMIndex < textmap.size()) {
                TextMap tm = textmap.get(currentTMIndex);
                if (tm.type == Text.Type.TextRef) {
                    startIndex = currentTMIndex;
                    adjusted = true;
                } else {
                    currentTMIndex++;
                }
            }
        }
        if (adjusted) {
            annotationStart.row = startIndex;           // point to first row after comments and bookmarks
        }

        currentTMIndex = saveTMIndex;

        // Find text that contains the end of the annotations
        while (endIndex == -1 && currentTMIndex < textmap.size()) {
            TextMap tm = textmap.get(currentTMIndex);
            if (annotationEnd.row <= tm.tabEndrow) {
                if ( tm.type == Text.Type.TextRef) {
                    endIndex = currentTMIndex;
                } else {
                    break;          // annotation ends in a comment or bookmark
                }
            } else {
                currentTMIndex++;
            }
        }

        if (endIndex == -1) {           // annotation ends in a comment or bookmark, find previous textref
            currentTMIndex--;
            while (endIndex == -1 && currentTMIndex >= 0) {
                TextMap tm = textmap.get(currentTMIndex);
                if (tm.type == Text.Type.TextRef) {
                    endIndex = currentTMIndex;
                } else {
                    currentTMIndex--;
                }
            }
        }

        if (endIndex == -1 || startIndex == -1 || endIndex < startIndex) {
            return;         // annotation is completely within comments and bookmarks
        }
        // Generate annotation chunks
        // Note: startRow, startCol, endRow, endCol are relative to a given text block and not to the entire text in tab
        // In contrast annotationStart, annotationEnd and tm values are relative to start of entire text in the tab.
        for (int i = startIndex; i<=endIndex; i++) {
            TextMap tm = textmap.get(i);
            if (tm.type == Text.Type.TextRef) {
                if (i == startIndex) {
                    startRow = annotationStart.row - tm.tabStartrow;
                    startCol = annotationStart.col;
                    if (annotationEnd.row <= tm.tabEndrow) {
                        endRow = annotationEnd.row - tm.tabStartrow;  // annotation fits completely in this block
                        endCol = annotationEnd.col;
                    } else {
                        endRow = tm.tabEndrow - tm.tabStartrow;   // annotation goes to end of this block and continues onto next
                        endCol = lines.get(tm.tabEndrow).length() - 1;     // go to last column of last line of block
                    }
                } else if (i != endIndex) {   // annotation spans entire block
                    startRow = 0;
                    startCol = 0;
                    endRow = tm.tabEndrow - tm.tabStartrow;   // last row of text block
                    endCol = lines.get(tm.tabEndrow).length() - 1;  // last col of last row of text block
                } else {        // annotation finishes up in this block
                    startRow = 0;   // starts from beginning of block
                    startCol = 0;
                    endRow = annotationEnd.row - tm.tabStartrow;
                    endCol = annotationEnd.col;
                }

                if (startRow < 0 || endRow < 0) {
                    startRow = startRow + endRow - endRow;
                }
                startRow += tm.textStartrow;
                endRow += tm.textStartrow;
                TextPoint _annotationStart = new TextPoint(startRow, startCol);
                TextPoint _annotationEnd = new TextPoint(endRow, endCol);
                appstate.addTextMarker(tm.filename, _annotationStart, _annotationEnd);
            }
        }
    }

    private int fixupRow (int row, TextMap tm) {
        return row - tm.tabStartrow;
    }

    public boolean selectionExists()
    {
        TextPoint start = getSelectionStart();
        TextPoint end = getSelectionEnd();
        return start.row != end.row || start.col != end.col;
    }

    public TextPoint getSelectionStart () {
        return ac.getSelectionStart();
    }

    public TextPoint getSelectionEnd () {
        return ac.getSelectionEnd();
    }

    public void setTextFocus()          // called when user does a Ctrl + 2
    {
        //this.grabFocus();
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                giveFocus();
            }
        });
    }

    boolean isModificationAllowed()
    {
        boolean rc = true;
        if (!appstate.getAllowModification())               // if modifications are not allowed
        {
            ac.highlight(appstate.getTextStart(textmap), appstate.getTextEnd(textmap));       // so that the selection won't disappear while dialog box is up.

            appstate.getAnnotater().allowModification();

            ac.highlight(appstate.getTextStart(textmap), appstate.getTextEnd(textmap));       // need to set highlight again because focus returned to text area

            return rc;
        }

        return rc;
    }


}
