package sita;

import org.sanskritlibrary.catalogs.Annotated;
import org.sanskritlibrary.catalogs.ApplicationState;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;

// Manages the main menu bar of the application
public class FeaturesMenuBar extends JMenuBar
{
    ApplicationState appstate;
// Text menu
    private JMenu textsMenu;
    private JMenuItem chooseText;
    private JMenuItem nextText;
    private JMenuItem prevText;
    private JMenuItem setDefault;
    private JMenuItem configurationDialog;
    private JMenuItem updateVirtualFiles;
    private JMenuItem closeApplication;

// Analysis menu
    private JMenu analysisMenu;
    private JMenu analysisOptions;
    private JRadioButtonMenuItem currentManuscript;
    private JRadioButtonMenuItem allManuscripts;
    private JMenuItem verseLookup;
    private JMenuItem annotationsReport;
    private JMenuItem commentsReport;
    private JMenuItem differenceReport;
    private JMenuItem searchText;

// View menu
    private JMenu viewMenu;
    private JMenuItem reorient;
    private JMenuItem zoomIn;
    private JMenuItem zoomOut;
    private JMenu changeTextEncoding;
    private JMenuItem devanagariEncoding;
    private JMenuItem romanEncoding;
    private JMenuItem teluguEncoding;
    private JMenuItem slp1Encoding;
    private JMenuItem increaseFontSize;
    private JMenuItem decreaseFontSize;
    private JMenuItem showBookmarks;
    private JMenuItem appearance;
    private JMenuItem refresh;
    private JMenuItem fileInformation;
    private JMenuItem gotoLine;
    private JMenuItem eliza;

// Annotation menu
    private JMenu annotationsMenu;
    private JMenuItem newAnnotation;
    private JMenuItem searchAreaAnnotation;
    private JMenuItem textAreaAnnotation;
    private JMenuItem commentsAreaAnnotation;
    private JMenuItem nextAnnotation;
    private JMenuItem nextAnnotationNamed;
    private JMenuItem removeAnnotation;
    private JMenuItem swapBracketsAnnotation;
    private JMenuItem cannedCommentAnnotation;
    private JCheckBoxMenuItem reviewOnly;

// Help menu
    private JMenu helpMenu;
    private JMenuItem documentation;
    private JMenuItem about;

    public FeaturesMenuBar(ApplicationState appstate, ActionListener al, ItemListener il)
    {
        this.appstate = appstate;
        Annotated a = appstate.getAnnotater();          // used to call Main window to get actions for accelerator keys

        // ******************************* Texts menu
        textsMenu = new JMenu("Texts");
        textsMenu.addItemListener(il);
        this.add(textsMenu);

        chooseText = new JMenuItem("Choose Manuscript");
        chooseText.setActionCommand("T");
        chooseText.setAccelerator(a.getAccelerator("T"));
        chooseText.addActionListener(al);
        textsMenu.add(chooseText);

        nextText = new JMenuItem("Next page");
        nextText.setActionCommand("N");
        nextText.setAccelerator(a.getAccelerator("N"));
        nextText.addActionListener(al);
        textsMenu.add(nextText);

        prevText = new JMenuItem("Previous page");
        prevText.setActionCommand("P");
        prevText.setAccelerator(a.getAccelerator("P"));
        prevText.addActionListener(al);
        textsMenu.add(prevText);

        configurationDialog = new JMenuItem("Configuration");
        configurationDialog.setActionCommand("configure");
        configurationDialog.addActionListener(al);
        textsMenu.add(configurationDialog);

        updateVirtualFiles = new JMenuItem("Update virtual files");
        updateVirtualFiles.setActionCommand("updateVirtual");
        updateVirtualFiles.addActionListener(al);
        textsMenu.add(updateVirtualFiles);

        setDefault = new JMenuItem("Save defaults");
        setDefault.setActionCommand("S");
        setDefault.setAccelerator(a.getAccelerator("S"));
        setDefault.addActionListener(al);
        textsMenu.add(setDefault);

        if (!appstate.getCanCloseWindow())      // if windows close decoration has been disabled
        {
            closeApplication = new JMenuItem("Close application");
            closeApplication.setActionCommand("closeapp");
            //closeApplication.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
            closeApplication.addActionListener(al);
            textsMenu.add(closeApplication);
        }

        // *********************** View menu
        viewMenu = new JMenu("View");
        viewMenu.addItemListener(il);
        this.add(viewMenu);

        // Add command to rotate display
        reorient = new JMenuItem("Rotate display");
        reorient.setActionCommand("D");
        reorient.setAccelerator(a.getAccelerator("D"));
        reorient.addActionListener(al);
        viewMenu.add(reorient);

        zoomIn = new JMenuItem("Zoom in");
        zoomIn.setActionCommand("+");
        zoomIn.addActionListener(al);
        zoomIn.setAccelerator(a.getAccelerator("+"));
        viewMenu.add(zoomIn);

        zoomOut = new JMenuItem("Zoom out");
        zoomOut.setActionCommand("-");
        zoomOut.addActionListener(al);
        zoomOut.setAccelerator(a.getAccelerator("-"));
        viewMenu.add(zoomOut);

        // Add sub menu
        changeTextEncoding = new JMenu("Change text encoding");
        viewMenu.add(changeTextEncoding);

        // Devanagari encoding
        devanagariEncoding = new JMenuItem("Devanagari");
        devanagariEncoding.setActionCommand("encodingdeva");
        devanagariEncoding.addActionListener(al);
        changeTextEncoding.add(devanagariEncoding);

        // Roman encoding
        romanEncoding = new JMenuItem("Roman");
        romanEncoding.setActionCommand("encodingroman");
        romanEncoding.addActionListener(al);
        changeTextEncoding.add(romanEncoding);

        // Telugu encoding
        teluguEncoding = new JMenuItem("Telugu");
        teluguEncoding.setActionCommand("encodingtelugu");
        teluguEncoding.addActionListener(al);
        changeTextEncoding.add(teluguEncoding);

        // SLP1 encoding
        slp1Encoding = new JMenuItem("SLP1");
        slp1Encoding.setActionCommand("encodingslp1");
        slp1Encoding.addActionListener(al);
        changeTextEncoding.add(slp1Encoding);

        // Increase font size
        increaseFontSize = new JMenuItem("Increase font size");
        increaseFontSize.setActionCommand("increaseFontSize");
        increaseFontSize.addActionListener(al);
        viewMenu.add(increaseFontSize);

        // Decrease font size
        decreaseFontSize = new JMenuItem("Decrease font size");
        decreaseFontSize.setActionCommand("decreaseFontSize");
        decreaseFontSize.addActionListener(al);
        viewMenu.add(decreaseFontSize);


        showBookmarks = new JMenuItem("Show bookmarks");
        showBookmarks.setActionCommand("B");
        showBookmarks.addActionListener(al);
        showBookmarks.setAccelerator(a.getAccelerator("B"));
        viewMenu.add(showBookmarks);

        appearance = new JMenuItem("Change marker color");
        appearance.setActionCommand("appearance");
        appearance.addActionListener(al);
        viewMenu.add(appearance);

        refresh = new JMenuItem("Refresh screen");
        refresh.setActionCommand("R");
        refresh.setAccelerator(a.getAccelerator("R"));
        refresh.addActionListener(al);
        viewMenu.add(refresh);

        fileInformation = new JMenuItem("Show file information");
        fileInformation.setActionCommand("I");
        fileInformation.setAccelerator(a.getAccelerator("I"));
        fileInformation.addActionListener(al);
        viewMenu.add(fileInformation);

        gotoLine = new JMenuItem("Go to line ...");
        gotoLine.setActionCommand("G");
        gotoLine.setAccelerator(a.getAccelerator("G"));
        gotoLine.addActionListener(al);
        viewMenu.add(gotoLine);

        eliza = new JMenuItem("Talk to Dr. Scharf");
        eliza.setActionCommand("eliza");
        eliza.addActionListener(al);
        //viewMenu.add(eliza);

        // ******************** Annotations menu
        annotationsMenu = new JMenu("Annotations");
        annotationsMenu.addItemListener(il);
        this.add(annotationsMenu);

        // New annotation
        newAnnotation = new JMenuItem("New annotation");
        newAnnotation.setActionCommand("A");
        newAnnotation.setAccelerator(a.getAccelerator("A"));
        newAnnotation.addActionListener(al);
        annotationsMenu.add(newAnnotation);

        // Search area
        searchAreaAnnotation = new JMenuItem("Search area");
        searchAreaAnnotation.setActionCommand("1");
        searchAreaAnnotation.setAccelerator(a.getAccelerator("1"));
        searchAreaAnnotation.addActionListener(al);
        annotationsMenu.add(searchAreaAnnotation);

        // Text area
        textAreaAnnotation = new JMenuItem("Text area");
        textAreaAnnotation.setActionCommand("2");
        textAreaAnnotation.setAccelerator(a.getAccelerator("2"));
        textAreaAnnotation.addActionListener(al);
        annotationsMenu.add(textAreaAnnotation);

        // Comments area
        commentsAreaAnnotation = new JMenuItem("Comments area");
        commentsAreaAnnotation.setActionCommand("3");
        commentsAreaAnnotation.setAccelerator(a.getAccelerator("3"));
        commentsAreaAnnotation.addActionListener(al);
        annotationsMenu.add(commentsAreaAnnotation);

        // Next annotation
        nextAnnotation = new JMenuItem("Next annotation");
        nextAnnotation.setActionCommand(">");
        nextAnnotation.setAccelerator(a.getAccelerator(">"));
        nextAnnotation.addActionListener(al);
        annotationsMenu.add(nextAnnotation);

        // Next annotation with popup
        nextAnnotationNamed = new JMenuItem("Choose annotation...");
        nextAnnotationNamed.setActionCommand("'");
        nextAnnotationNamed.setAccelerator(a.getAccelerator("'"));
        nextAnnotationNamed.addActionListener(al);
        annotationsMenu.add(nextAnnotationNamed);

        removeAnnotation = new JMenuItem("Remove annotation");
        removeAnnotation.setActionCommand("X");
        removeAnnotation.setAccelerator(a.getAccelerator("X"));
        removeAnnotation.addActionListener(al);
        annotationsMenu.add(removeAnnotation);

        swapBracketsAnnotation = new JMenuItem("Swap brackets");
        swapBracketsAnnotation.setActionCommand("[");
        swapBracketsAnnotation.setAccelerator(a.getAccelerator("["));
        swapBracketsAnnotation.addActionListener(al);
        annotationsMenu.add(swapBracketsAnnotation);

        cannedCommentAnnotation = new JMenuItem("Insert named comment");
        cannedCommentAnnotation.setActionCommand("/");
        cannedCommentAnnotation.setAccelerator(a.getAccelerator("/"));
        cannedCommentAnnotation.addActionListener(al);
        annotationsMenu.add(cannedCommentAnnotation);

        if (appstate.getShowReviewOnly()) {
            reviewOnly = new JCheckBoxMenuItem("Review only");
            reviewOnly.setActionCommand("reviewOnly");
            reviewOnly.setSelected(appstate.getReviewOnly());
            reviewOnly.addActionListener(al);
            annotationsMenu.add(reviewOnly);
        }

        // Analysis menu
        analysisMenu = new JMenu("Analysis");
        analysisMenu.addItemListener(il);
        this.add(analysisMenu);

        analysisOptions = new JMenu("Analysis options");
        analysisMenu.add(analysisOptions);

        currentManuscript = new JRadioButtonMenuItem("Current manuscript only", true);
        analysisOptions.add(currentManuscript);

        allManuscripts = new JRadioButtonMenuItem("All manuscripts", false);
        analysisOptions.add(allManuscripts);

        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(currentManuscript);
        buttonGroup.add(allManuscripts);

        verseLookup = new JMenuItem("Lookup verse");
        verseLookup.setActionCommand("lookup");
        verseLookup.addActionListener(al);
        analysisMenu.add(verseLookup);

        annotationsReport = new JMenuItem("Annotations report");
        annotationsReport.setActionCommand("annotationsreport");
        annotationsReport.addActionListener(al);
        analysisMenu.add(annotationsReport);

        commentsReport = new JMenuItem("Comments report");
        commentsReport.setActionCommand("commentsreport");
        commentsReport.addActionListener(al);
        analysisMenu.add(commentsReport);

        differenceReport = new JMenuItem("Difference report");
        differenceReport.setActionCommand("differencesreport");
        differenceReport.addActionListener(al);
        analysisMenu.add(differenceReport);

        searchText = new JMenuItem("Search for text");
        searchText.setActionCommand("searchtext");
        searchText.addActionListener(al);
        analysisMenu.add(searchText);

        // Help Menu
        helpMenu = new JMenu("Help");
        helpMenu.addItemListener(il);
        this.add(helpMenu);

        documentation = new JMenuItem("Help contents");
        documentation.setActionCommand("help");
        documentation.addActionListener(al);
        helpMenu.add(documentation);

        about = new JMenuItem("About");
        about.setActionCommand("about");
        about.addActionListener(al);
        helpMenu.add(about);
}

    // Called by the ItemChangeListener whenever a menu is pulled down
    public void enableCommands()
    {
        boolean isDisplaySuccessful = appstate.isDisplaySuccessful();       // i.e., there is a manuscript image and some text visible.

        // Text menu
        nextText.setEnabled(isDisplaySuccessful);
        prevText.setEnabled(isDisplaySuccessful);
        setDefault.setEnabled(isDisplaySuccessful);

        // View menu
        viewMenu.setEnabled(isDisplaySuccessful);

        // Analysis menu
        analysisMenu.setEnabled(isDisplaySuccessful);
        
        // Annotation menu
        annotationsMenu.setEnabled(isDisplaySuccessful);


        if (isDisplaySuccessful)
        {
            boolean haveAnnotations = appstate.getNumAnnotations() > 0;
            nextAnnotation.setEnabled(haveAnnotations);
            removeAnnotation.setEnabled(haveAnnotations);
            swapBracketsAnnotation.setEnabled(appstate.getLeftBracketExists() && appstate.getRightBracketExists());

            if (appstate.getNumberOfEncodings() > 1) {          // number of encodings possible for current tab
                changeTextEncoding.setEnabled(true);
                setEncodingEnabled(devanagariEncoding, "deva");
                setEncodingEnabled(romanEncoding, "roman");
                setEncodingEnabled(teluguEncoding, "telugu");
                setEncodingEnabled(slp1Encoding, "slp");
            }
            else {
               changeTextEncoding.setEnabled(false);
            }
        }
    }

    private void setEncodingEnabled(JMenuItem mi, String encoding)
    {
        String currentEncoding = appstate.getEncoding();

        if (currentEncoding != null)
        {
            if (currentEncoding.equals(encoding))
                mi.setVisible(false);
            else
                mi.setVisible(appstate.getEncodingExists(encoding));
        }
        else
            mi.setVisible(false);
    }

    public boolean currentManuscriptOnly()
    {
        return currentManuscript.isSelected();
    }

    public boolean reviewOnly() {
        return reviewOnly.isSelected();
    }
}
