package sita;

import java.awt.event.AdjustmentEvent;
import javax.swing.JComponent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JScrollBar;
import java.awt.event.AdjustmentListener;

public class ImageScrollPane extends JComponent implements AdjustmentListener
{
    ImagePanel imagePanel;
    JScrollBar horizontalScrollBar;
    JScrollBar verticalScrollBar;

    public ImageScrollPane(ImagePanel imagePanel)
    {
        imagePanel.setScroller(this);
        this.imagePanel = imagePanel;
        setLayout(new BorderLayout());
        add(imagePanel, BorderLayout.CENTER);

        /* new JScrollBar(orientation, value, extent, min, max)
         *     orientation: JScrollBar.HORIZONTAL or JScrollBar.VERTICAL
         *     value:       current value of the scroll bar (starts at min?)
         *     extent:      length of the knob (thumb)
         *     min:         minimum value that value can take
         *     max:         maximum value that value can take
        */
        horizontalScrollBar = new JScrollBar(JScrollBar.HORIZONTAL, 0, imagePanel.getDisplayWidth(), 0, imagePanel.getDisplayWidth());
        horizontalScrollBar.setUnitIncrement(10);
        horizontalScrollBar.addAdjustmentListener(this);
        add(horizontalScrollBar, BorderLayout.SOUTH);

        verticalScrollBar = new JScrollBar(JScrollBar.VERTICAL, 0, imagePanel.getDisplayHeight(), 0, imagePanel.getDisplayHeight());
        verticalScrollBar.setUnitIncrement(10);
        verticalScrollBar.addAdjustmentListener(this);
        add(verticalScrollBar, BorderLayout.EAST);

        int width = imagePanel.getWidth() + verticalScrollBar.getWidth();
        int height = imagePanel.getHeight() + horizontalScrollBar.getHeight();
        this.setSize(new Dimension(width, height));
        this.setPreferredSize(new Dimension(width, height));
    }

    public int getHorizontalScrollBarHeight()
    {
        return horizontalScrollBar.getHeight();
    }

    public int getHorizontalValue()
    {
        return horizontalScrollBar.getValue();
    }

    public void setHorizontalValue(int n)
    {
        horizontalScrollBar.setValue(n);
    }

    public int getVerticalValue()
    {
        return verticalScrollBar.getValue();
    }

    public void setVerticalValue(int n)
    {
        verticalScrollBar.setValue(n);
    }

    public void setMinimumHorizontal(int n)
    {
        horizontalScrollBar.setMinimum(n);
    }

    public void setMaximumHorizontal(int n)     // n is the width of the reduced image being displayed
    {
        horizontalScrollBar.setMaximum(n);
        int displayWidth = imagePanel.getDisplayWidth();
        if (n >= displayWidth)     // if horizontal extent of image to big to be seen all at once, be sure that extent is equal to the display width
            horizontalScrollBar.setVisibleAmount(displayWidth);
        else
            horizontalScrollBar.setVisibleAmount(n);        // thumb is entire bar
    }

    public void setMinimumVertical(int n)
    {
        verticalScrollBar.setMinimum(n);
    }

    public void setMaximumVertical(int n)
    {
        verticalScrollBar.setMaximum(n);
        int displayHeight = imagePanel.getDisplayHeight();
        if (n >= displayHeight)     // if vertical extent of image to big to be seen all at once, be sure that extent is equal to the display height
            verticalScrollBar.setVisibleAmount(displayHeight);
        else
            verticalScrollBar.setVisibleAmount(n);        // thumb is entire bar
    }

    public void setExtentHorizontal(int n)
    {
        horizontalScrollBar.setVisibleAmount(n);
    }

    public void setExtentVertical(int n)
    {
        verticalScrollBar.setVisibleAmount(n);
    }


    public void adjustmentValueChanged(AdjustmentEvent e)
    {
        if (e.getSource() == horizontalScrollBar)
        {
            int n = e.getValue();
            imagePanel.adjustHorizontal(n);
        }
        else if (e.getSource() == verticalScrollBar)
        {
            int n = e.getValue();
            imagePanel.adjustVertical(n);
        }
    }

    @Override
    public String toString()
    {
        return String.format("H: value=%d, min=%d, max=%d, extent=%d\nV: value=%d, min=%d, max=%d, extent=%d",
                                horizontalScrollBar.getValue(),
                                horizontalScrollBar.getMinimum(),
                                horizontalScrollBar.getMaximum(),
                                horizontalScrollBar.getVisibleAmount(),

                                verticalScrollBar.getValue(),
                                verticalScrollBar.getMinimum(),
                                verticalScrollBar.getMaximum(),
                                verticalScrollBar.getVisibleAmount()
                                );
    }
}
