package sita;

import org.sanskritlibrary.catalogs.VerseAnnotationEx;
import org.sanskritlibrary.catalogs.ApplicationState;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.DefaultListModel;
import javax.swing.InputMap;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.sanskritlibrary.Regex;

public class SearchTextDialog extends JDialog implements ListSelectionListener
{
    ArrayList<VerseAnnotationEx> foundImages;          // contains the image(s) that the selected verse number are along with the annotation with the longest highlight of the selected verse

    ApplicationState appstate;
    boolean useCurrentManuscript;
    boolean ignoreValueChanged;
    JList resultsList;
    DefaultListModel resultsListModel;
    JScrollPane resultsListScrollPane;
    JTextArea query;
    Component _this;
    int lastResultSelected = -1;

    public SearchTextDialog(ApplicationState appstate)
    {
        this.appstate = appstate;
        foundImages = new ArrayList<VerseAnnotationEx>();

        setLayout(new BorderLayout());

        query = new JTextArea();
        query.setPreferredSize(new Dimension(180, 20));
        InputMap im = query.getInputMap();
        ActionMap am = query.getActionMap();
        im.put(KeyStroke.getKeyStroke("ENTER"), "search");
        am.put("search", new SearchAction());
        appstate.getAnnotater().setupInputActions(im, am);

        add(query, BorderLayout.NORTH);

        resultsListModel = new DefaultListModel();
        resultsList = new JList(resultsListModel);
        resultsList.addListSelectionListener(this);
        resultsList.setVisibleRowCount(10);
        //resultsList.setPreferredSize(new Dimension(180, 100));
        resultsListScrollPane = new JScrollPane(resultsList);
        resultsListScrollPane.setPreferredSize(new Dimension(180, 100));
        resultsListScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        add(resultsListScrollPane, BorderLayout.CENTER);

        _this = this;

        setBounds(0, 0, 250, 280);
        Utils.upperRightScreen(this);
        setModal(false);
        setAlwaysOnTop(true);
    }

    public void display(boolean useCurrentManuscript)
    {
        useCurrentManuscript = true;
        this.useCurrentManuscript = useCurrentManuscript;
        setVisible(true);
    }

    public void valueChanged(ListSelectionEvent e)
    {
        if (ignoreValueChanged)
            return;

        int n = resultsList.getSelectedIndex();             // to avoid this getting called twice.
        if (n == lastResultSelected)
            return;
        lastResultSelected = n;
        Utils.displayWaitCursor(this);
        ignoreValueChanged = true;
        appstate.getAnnotater().saveState();
        try
        {
            String rootDirectory = appstate.getManuscriptRootDirectory();

            String manuscriptDirectory = foundImages.get(n).manuscriptDirectory;      // get the directory that contains the desired page
            String lastPage = (String) resultsList.getSelectedValue();
            Regex re = new Regex(lastPage, "([^\\\\/]+\\.jpg)");
            re.find();
            String abbrev = re.getGroup(1);
            String pageName = appstate.getImageFilename(manuscriptDirectory, abbrev);
            String imagefile = String.format("%s/%s/%s", rootDirectory, manuscriptDirectory, pageName);

            re = new Regex(abbrev, "^(.*)\\.[^.]+$");
            re.find();
            appstate.getAnnotater().setCurrentPage(manuscriptDirectory, re.getGroup(1));
            appstate.setMoveToAnnotationNumber(this.foundImages.get(n).annotationNumbers.get(0));
            appstate.getAnnotater().displayManuscript(imagefile);
            appstate.getAnnotater().bringToFront();
        } catch (Exception ex)
        {
            Utils.displayError(ex);
        }
        ignoreValueChanged = false;
        Utils.displayDefaultCursor(this);
    }

    private class SearchAction extends AbstractAction
    {
        public void actionPerformed(ActionEvent ev)
        {
            ignoreValueChanged = true;

            String text = query.getText();

            try
            {
                appstate.searchForText(useCurrentManuscript, text, foundImages);
            }
            catch (Exception ex)
            {
                Logger.writeLog(text);
            }

            if (foundImages.size() > 0)
            {
                int n = 0;
                resultsListModel.clear();

                for (VerseAnnotationEx va : foundImages)
                {
                    Regex re = new Regex(va.filename, "(\\p{Nd}+)\\.xml$");
                    re.find();
                    String name = re.getGroup(1) + ".jpg";

                    if (useCurrentManuscript)
                        resultsListModel.add(n, name);
                    else
                        resultsListModel.add(n, String.format("%s/%s\n", va.manuscriptDirectory, name));

                    n++;
                }
            }
            else
                JOptionPane.showMessageDialog(_this, "Not found");

            ignoreValueChanged = false;
        }
    }
}
